module ML.HS2MLPrelude ( hsPrelude, mosmlPrelude, hsIds ) where

import Phrac.Error

hsSrc = 
    [ ("head", "List __a -> __a")
    , ("tail", "List __a -> List __a") 
    , ("null", "List __a -> Bool")
    , ("append", "List __a -> List __a -> List __a")
    , ("fix", "(__a -> __a) -> __a")
    , ("primIntEq", "Int -> Int -> Bool")
    , ("primCharEq", "Char -> Char -> Bool")
    , ("primIntAdd", "Int -> Int -> Int")
    , ("(&&)", "Bool -> Bool -> Bool")
    , ("(||)", "Bool -> Bool -> Bool")
    ]

dataSrc = [ "data List __a = Cons __a (List __a) | Nil;"
          , "data Bool = True | False;"
          ]
hsPrelude = 
    unlines (map gen hsSrc) ++ unlines dataSrc
    where gen (id, t) = id ++ " :: " ++ t ++ "; " ++ id ++ " = " ++ id ++ ";"

hsIds = map id hsSrc
    where id (('(':rest), _) = 
                        case reverse rest of
                             (')':rest') -> reverse rest'
                             _ -> panic ("identifier starts with a '(' but does " ++
                                         "not end with a ')'")
          id (x,_) = x

mosmlPrelude = unlines
           [ "Meta.quietdec := true;"
           , "val cz_head = hd;"
           , "val cz_tail = tl;"
           , "val cz_null = null;"
           , "fun cz_append l1 l2 = l1 @ l2;"
           , "local datatype 'a fix = FUN of 'a fix -> 'a "
           , "      fun NUF (FUN x) = x "
           , "in fun cz_fix f = (fn x => (fn y => f((NUF x) x) y)) "
           , "              (FUN(fn x => (fn y => f((NUF x) x) y))) end;"
           , "fun cz_primIntEq i j = (i:int) = j;"
           , "fun cz_primCharEq c1 c2 = (c1:char) = c2;"
           , "fun cz_primIntAdd i j = (i:int) + j;"
           ]
