module ML.Mosml where

import ML.AST
import ML.HS2MLPrelude
import Phrac.Pretty
import Phrac.Error
import Process

type MosmlCmd = String

mosmlArgs = ["-quietdec", "-liberal", "-P", "full"]


errorPrefix = "! Toplevel input"
errorPrefix2 = "> val it = () : unit\n- " ++ errorPrefix

runMosml :: Prog -> MosmlCmd -> IO String
runMosml prog cmd =
    do debug ("Moscow ML command: " ++ cmd)
       let src = show $ ppr prog
       let fullSrc = mosmlPrelude ++ src
       debug ("Pretty printed (full) source code:\n" ++ fullSrc)
       debug "Now running Moscow ML"
       (out, err, pid) <- popen cmd mosmlArgs (Just fullSrc)
       if not (null err) || out `startswith` errorPrefix || 
          out `startswith` errorPrefix2
          then panic ("Moscow ML failed!!\n\n==> Input:\n" ++ 
                      (number fullSrc) ++
                      "\n\n==>Stdout:\n" ++ out ++ "\n\n==>Stderr:\n" ++ err)
          else debug ("Moscow ML finished successfully, output:\n" ++ out)
       return out

-- adds line numbers
number :: String -> String
number s = let l = lines s
               l' = map (\ (i,x) -> align (length l) i ++ ": " ++ x) (zip [1..] l)
               in unlines l'
    where align n i = let s = show i
                          in take (length (show n) - length s) (repeat ' ') ++ s

startswith s pref = 
    let n = length pref
        in pref == (take n s)

