module ASTTest where

import ML.AST
import ML.Identifiers
import ML.Mosml
import Phrac.Pretty
import Phrac.Error

import TestFramework

{-
    signature SIG = sig val g : 'a -> 'b -> 'a end

    rec
      functor F(X: sig end) : SIG =
        struct
          val g = \y -> \z ->
                open (pack F(struct end) as SIG) as X : SIG in
                  let q = X.g true in
                  let r = X.g 0 in y
        end

    structure X = F(struct end)
-}

sig = SigId "SIG"
g = ValId "g"
f = FunId "F"
x = StrId "X"
a = SimTyvar "'a"
b = SimTyvar "'b"
y = CoreId "y"
z = CoreId "z"
q = CoreId "q"
r = CoreId "r"
true = CoreId "true"

emptySig = SigBod []
signatureSIG = SigBind sig $ SigBod [ValSpec g (ValTyp [a] 
                                                (tyFun [SimTypTyvar a,
                                                        SimTypTyvar b,
                                                        SimTypTyvar a]))]

emptyStruct = StrBod []

functorF = Rfun f [(x, emptySig)] (SigVar sig) (StrBod [valG])

valG = ValDef g Nothing expG
expG = LamExp y
       (LamExp z
        (OpenExp (PackExp (FunctorApp f [emptyStruct]) (SigVar sig)) x (SigVar sig)
         (LetExp q Nothing (AppExp (QModVarExp x g) (CoreVarExp true)) -- in
          (LetExp r Nothing (AppExp (QModVarExp x g) (IntExp 0))        -- in
           (CoreVarExp y)))))

mainExp = 
    OpenExp (PackExp (FunctorApp f [emptyStruct]) (SigVar sig)) x (SigVar sig)
    (AppExp (AppExp (QModVarExp x g) (IntExp 42)) (IntExp 0))

prog = Prog [] [signatureSIG] (Rfuns [functorF]) mainExp

evalProg = do res <- catchPE $ runMosml prog "mosml"
              case res of
                Left e -> assertFailure ("running Moscow ML failed: " ++ show e)
                Right x -> return x

$(tests "aSTTests" [d|

 test1 = 
     do writeFile "test.sml" (showPpr prog)
        out <- evalProg
        assertEqual "42\n" out

 |])